% ------------------------------------------------------------------------
% [1]: Hichem Abdellali, Zoltan Kato ;
% Absolute and Relative Pose Estimation of a Multi-View Camera System using
% 2D-3D Line Pairs and Vertical Direction;
% 2018 International Conference on Digital Image Computing.
% All rights reserved.
% ------------------------------------------------------------------------

% published: 2020-04-29,
% Authors: Hichem Abdellali, Zoltan Kato  using Matlab (R2018b)
% All rights reserved.


% Important!
% ------------------------------------------------------------------------
% Please cite our [1] publication whenever you use the implementation.


% Introduction
% ------------------------------------------------------------------------
% This software package contains the full algorithm as described in paper [1].




% Installation
% ------------------------------------------------------------------------

% This package does not require any installation. Package contains all
% necessary functions. the solver needs endpoints from 2D-3D line
% coorespandances (an example is provided),


% ------------------------------------------------------------------------
% -------------------------------- NOTE ----------------------------------
% ------------------------------------------------------------------------
% we assume a calibrated perspective camera, and we work with 
% the equivalent normalized image coordinates. 
% If our camera is a calibrated camera and we have a matrix M of size 3xn 
% which contains the 2D coordinates with homogenous 
% coordinate representation in the image plane then we can normalize 
% the coordinates as follows:

% temp = inv(K) * M;
% for i=1:n
%   M_normalized(:,i) = temp(:,i) / temp(3,i)
% end

% ------------------------------------------------------------------------
% ---------------------------- Main input --------------------------------
% ------------------------------------------------------------------------
% - input.Camera(i).Line2D.start: normalized coordinates of the image line 
% segment startpoints.
% - input.Camera(i).Line2D.end: normalized coordinates of the image line 
% segment endpoints.
% - input.Camera(i).vertical_direction: The vertical direction provided 
% as [ angle of Ry in radian, angle of Rz in angle]  
% - input.Camera(1).Line3D.start:  start points of the 3D line segments 
% in the world coordinates.
% - input.Camera(1).Line3D.end:  end points of the 3D line segments 
% in the world coordinates.

% - lineNum: number of 2D-3D line pairs to be used.
% - iterations: number of iteration that the direct solver will do


% ------------------------------------------------------------------------
% ----------------------------- output -----------------------------------
% ------------------------------------------------------------------------
% poses : a cell which contains a list of poses of 4*4 matrix
% poses{1,1} acts from the world coordinate system to the reference 
% camera ('Absolute pose').
% poses{1,2:camNum} acts from the reference camera to the other 
% cameras ('Relative Poses').

% Note that the algorithm requires at least 2 cameras to work! please check
% Input.mat

% ------------------------------------------------------------------------
% ------------------------------ Tests -----------------------------------
% ------------------------------------------------------------------------
% run this script "HOW_TO_RUN.m". an example will be loaded from Input.mat
% it contains a configuration of 5 cameras (first one is the reference),
% and 60 2D-3D line correspandances, 

dbstop if error
restoredefaultpath
load('Input.mat');

addpath checking
addpath DS
addpath FormatingData
addpath LS
addpath Translation
addpath Solver


iterations = 50;
camNum=2;
lineNum=60;

[Pose] = mv_pose_vd(input,camNum,lineNum,iterations);



